/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * Copyright (C) 2011 matt mooney <mfm@muteddisk.com>
 *               2005-2007 Takahiro Hirofuchi
 * Copyright (C) 2015-2016 Samsung Electronics
 *               Igor Kotrasinski <i.kotrasinsk@samsung.com>
 *               Krzysztof Opasiak <k.opasiak@samsung.com>
 */

#ifndef __USBIP_HOST_DRIVER_H
#define __USBIP_HOST_DRIVER_H

#include <stdint.h>
#include "usbip_common.h"
#include "list.h"
#include "usbip_host_common.h"

extern struct usbip_host_driver host_driver;

#endif /* __USBIP_HOST_DRIVER_H */
